#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Image_Surface.H>
#include <FL/fl_draw.H>
#include <cassert>

Fl_Image *bg_image = 0;

class MyWindow : public Fl_Double_Window
{
typedef Fl_Double_Window Inherited;
public:
	MyWindow( int w_, int h_, const char *l_ = 0 ) :
		Inherited( w_, h_, l_ )
	{
	}
protected:
	void draw()
	{
		bg_image->draw( 0, 0 );
	}
};

Fl_Image *prepare_bg( int W, int H )
{
	printf( "prepare_bg %d x %d", W, H );
	Fl_Image_Surface *img_surf = new Fl_Image_Surface( W, H );
	assert( img_surf );
	img_surf->set_current();

	// background
	fl_rectf( 0, 0, W, H, FL_BLUE );

	// top 1/4 height random color
	fl_rectf( 0, 0, W, H / 4, fl_rgb_color( random() % 256, random() % 256, random() % 256  ) );

	// bottom 1/4 height random color
	fl_rectf( 0, H - H / 4, W, H / 4, fl_rgb_color( random() % 256, random() % 256, random() % 256  ) );

	Fl_RGB_Image *image = img_surf->image();
	delete img_surf;
	Fl_Display_Device::display_device()->set_current();
	printf( " ==> image %d x %d\n", image->w(), image->h() );
	return image;
}

static void cb_new_image( void *d_ )
{
	MyWindow *win = (MyWindow *)d_;
	delete bg_image;
	bg_image = prepare_bg( win->w(), win->h() );
	win->redraw();
	Fl::repeat_timeout( 2., cb_new_image, d_ );
}

int main( int argc_, char *argv_[] )
{
	const int W = 400;
	const int H = 300;
	bg_image = prepare_bg( W, H );
	MyWindow win( W, H );
	win.end();
	win.show();
	Fl::add_timeout( 2, cb_new_image, &win );
	return Fl::run();
}
